#include <avr/pgmspace.h>

// this constant won't change:
const int  buttonPin = 12;    // the pin that the pushbutton is attached to

// Variables will change:
unsigned int buttonState = 0;         // current state of the button
unsigned int lastButtonState = 0;     // previous state of the button

//The PROGMEM is unable to store non-const data
//SAVE SOME STUFF INTO FLASH MEMORY ...

//A) Single unsigned integer (0 to 65535) 
unsigned int OneInteger_RAM;
PROGMEM prog_uint16_t OneInteger_FLASH = 100; //prog_uint16_t -> an unsigned int (2 bytes) 0 to 65,535


//B) Array of bytes (0,255) 
byte ArrayOfBytes_RAM[] = {0, 0};
PROGMEM prog_uchar ArrayOfBytes_FLASH[] = {5, 44}; //prog_uchar -> an unsigned char (1 byte) 0 to 255


//C) String (Char array)
unsigned char LongMessage_RAM[] = {""};
PROGMEM prog_uchar LongMessage_FLASH[] = {"This is a long message string"}; //prog_uchar -> an unsigned char (1 byte) 0 to 255


//D) Array of integers (0 to 65535)
unsigned int ArrayOfIntegers_RAM[] = {0, 0, 0, 0, 0};
PROGMEM prog_uint16_t ArrayOfIntegers_FLASH[] = {61153, 32, 16843, 10, 11234}; //prog_uint16_t -> an unsigned int (2 bytes) 0 to 65,535



void ReadFromFLASH(){
  
  // read back a 2-byte int
  OneInteger_RAM = pgm_read_word(&OneInteger_FLASH);  
  
  // read back a byte int
  ArrayOfBytes_RAM[0] = pgm_read_byte(&ArrayOfBytes_FLASH[0]); 
  
  // read back a byte int (only the first letter of the long string)
  LongMessage_RAM[0] = pgm_read_byte(&LongMessage_FLASH[0]);
  
  // read back a 2-byte int
  ArrayOfIntegers_RAM[0] = pgm_read_word(&ArrayOfIntegers_FLASH[0]);
  
}


void PrintProgString(const prog_uchar str[])
{
  char c;
  if(!str) return;
  while((c = pgm_read_byte(str++)))
    Serial.print(c,BYTE);
}


void setup() {
  // initialize the button pin as a input:
  pinMode(buttonPin, INPUT);

  // initialize serial communication:
  Serial.begin(9600);  
}


void loop() {
  // read the pushbutton input pin:
  buttonState = digitalRead(buttonPin);

  // compare the buttonState to its previous state
  if (buttonState != lastButtonState) {
    // if the state has changed, increment the counter
    if (buttonState == HIGH) {

      Serial.println("on");
      
      Serial.print("Initial OneInteger_RAM ");
      Serial.println(OneInteger_RAM, DEC);
      
      Serial.print("Initial ArrayOfBytes_RAM[0] ");
      Serial.println(ArrayOfBytes_RAM[0], DEC);
      
      Serial.print("Initial LongMessage_RAM[0] ");
      Serial.println(LongMessage_RAM[0], DEC); 
      
      Serial.print("Initial ArrayOfIntegers_RAM[0] ");
      Serial.println(ArrayOfIntegers_RAM[0], DEC);
      
      ReadFromFLASH();
      
      Serial.print("Restored value from flash OneInteger_RAM ");
      Serial.println(OneInteger_RAM, DEC);
   
      Serial.print("Restored value from flash ArrayOfBytes_RAM[0] ");
      Serial.println(ArrayOfBytes_RAM[0], DEC);   
      
      Serial.print("Restored value from flash LongMessage_RAM[0] ");
      Serial.println(LongMessage_RAM[0], BYTE); 
      
      Serial.print("Restored value from flash ArrayOfIntegers_RAM[0] ");
      Serial.println(ArrayOfIntegers_RAM[0], DEC); 
     
     PrintProgString(LongMessage_FLASH); 

    }
    else {
      // if the current state is LOW then the button
      // went from on to off:
      Serial.println("off");
    }
  }
  
  // save the current state as the last state,
  //for next time through the loop
  lastButtonState = buttonState;

 
}
